/*
 * VarStar - Search & report variable star records in AAVSO data file
 * Copyright (C) 2005-2007  Frederick (Rick) Burton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

// Star function definitions 
//
// Loads vallongtype.txt into memory allowing searching by multiple
// criteria. This file is a public domain file downloaded from
// AAVSO at http://www.aavso.org

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "starfuncs.h"

//--== initRecords() +==-------------------------------------------------------
//
// Call this function FIRST before loading records
//
void initRecords()
{
	filename[0] = '\0';
	rptname[0]  = '\0';
	//starFile = NULL;
	//starOut  = NULL;

	topPtr     = NULL;
	lastPtr    = NULL;
	topResPtr  = NULL;
	resPtr     = NULL;
	lastResPtr = NULL;
	is_loaded   = FALSE;
	is_searched = FALSE;
	is_specific = FALSE;
	begin = FALSE;
	end	  = FALSE;

	minHr   = 0;
	maxHr   = 23;
	minDeg  = -90;
	maxDeg  = 90;
	minDays = 0;
	maxDays = 10000;
	minMag  = 6;
	maxMag  = 7;
}

//--== Functions Setting Search Criteria Vars ==-------------------------------
//
// If values are in acceptable range, change. Otherwise, initRecords()
// or a previous set operation has already determined the range.
//
void setSchMinHr  (short int min_ra_hrs)
{
	if(min_ra_hrs >= 0 && min_ra_hrs <= 23)
		minHr = min_ra_hrs;
}
void setSchMaxHr  (short int max_ra_hrs)
{
	if(max_ra_hrs >= 0 && max_ra_hrs <= 23)
		maxHr = max_ra_hrs;
}
void setSchMinDeg (short int min_dec_deg)
{
	if(min_dec_deg >= -90 && min_dec_deg <= 90
		&& min_dec_deg <= maxDeg)
		minDeg = min_dec_deg;
}
void setSchMaxDeg (short int max_dec_deg)
{
	if(max_dec_deg >= -90 && max_dec_deg <= 90
		&& max_dec_deg >= minDeg)
		maxDeg = max_dec_deg;
}
void setSchMinDays(double min_days)
{
	if(min_days >= 0 && min_days <= maxDays)
		minDays = min_days;
}
void setSchMaxDays(double max_days)
{
	if(max_days > 0 && max_days >= minDays)
		maxDays = max_days;
}
void setSchMinMag (double min_mag)
{
	if(min_mag >= 0)
		minMag  = (float)min_mag;
}
void setSchMaxMag (double max_mag)
{
	if(max_mag >= 0)
		maxMag  = (float)max_mag;
}

//--== Functions Getting Search Criteria Vals ==-------------------------------
//
// Return values protected by ?
//
//== Does short int make sense on current hardware? ==//
short int getSchMinHr  () {return minHr;}
short int getSchMaxHr  () {return maxHr;}
short int getSchMinDeg () {return minDeg;}
short int getSchMaxDeg () {return maxDeg;}
void  getSchMinDays(char* mind) {sprintf(mind, "%12.4lf", minDays);}
void  getSchMaxDays(char* maxd) {sprintf(maxd, "%12.4lf", maxDays);}
void  getSchMinMag (char* minm) {sprintf(minm, "%12.4f",  minMag);}
void  getSchMaxMag (char* maxm) {sprintf(maxm, "%12.4f",  maxMag);}

//--== loadRecords() ==--------------------------------------------------------
//
// Create master linked list data nodes from passed file name
//
int loadRecords(const char* fname)
{
	int i = 0, j, k;
	// How long should buffer be?
	char linebuf[96];
	struct VarStarRecord* recPtr;

	// Remember the file name
	strcpy(filename, fname);
	ifptr = fopen(fname, "r");
	if(ifptr == NULL)
		return -1;

	// Destroy search result linked list if there is one
	if(is_searched)
	{
		delResults();
		is_searched = 0;
	}

	// Destroy master linked list if there is one
	if(is_loaded)
	{
		delRecords();
		is_loaded = FALSE;
	}

	// Check for executable hanging the load
	while (!feof(ifptr))
	{
		strcpy(linebuf, "");
		fgets(linebuf, 96, ifptr);


		// If buffer contains a valid record,
		// add a node to the master linked list
		if(isLineValidRecord(linebuf))
		{
			if(i == 0)
			{
				topPtr = malloc(sizeof(struct VarStarRecord));
				recPtr = topPtr;
			}
			else
			{
				recPtr->pNext = 
					malloc(sizeof(struct VarStarRecord));
				recPtr = recPtr->pNext;
			}
			
			// This prevents the CR char in DOS from
			// causing problems with the space padder
			k = (int)(strlen(linebuf));
			if(linebuf[k-2] == 13)       // v1.01: added -2
			{                            // 
				linebuf[k-2] = (char)10; // LF after final char
				linebuf[k-1] = '\0';     // NULL terminator
				k--;
			}

			// Pad short records with spaces, crop long records
			if(k != 76) // v1.01: less than becomes not equal
			{
				if(k < 76)
					for(j = k-1; j < 75; j++)
						linebuf[j] = ' ';
				                         // v1.01: added -2
				linebuf[75] = (char)10;  // LF after final char
				linebuf[76] = '\0';      // NULL terminator
			}

			// Raw record
			strcpy(recPtr->record, linebuf);
			// Record number
			recPtr->recnum = ++i;
			// R.A. hours
			setHours(recPtr);
			// R.A. minutes
			setMinutes(recPtr);
			// Declination degrees
			setDegrees(recPtr);
			// Name of star
			setName(recPtr);
			// Type of star
			setType(recPtr);
			// Range string of star
			setRange(recPtr);
			// Maximum magnitude
			setMaxMag(recPtr);
			// Minumum magnitude
			setMinMag(recPtr);
			// Period in days
			setDays(recPtr);
			// Desc of star
			setDesc(recPtr);
		}

		if(feof(ifptr))
				break;
	}

	// If records were loaded, set vars
	if(i > 0)
	{
		is_loaded = TRUE;
		recPtr->pNext = NULL;
		clearSearch();
	}
	else
		is_loaded = FALSE;

	fclose(ifptr);
	return i;
}


//--== saveReport() ==---------------------------------------------------------
//
// Save current search results as a comma separated file with column headers
//
int saveReport (const char* fname)
{
	struct VarStarResult* rPtr = topResPtr;
	int i = 0;

	if(!is_loaded)
		return 0;

	// ".csv" appended to passed file name if necessary
	strcpy(rptname, fname);
	if(strstr(fname, ".csv") == NULL);
		strcat(rptname, ".csv");

	ofptr = fopen(rptname, "w");

	if(ofptr == NULL)
		return -1;	// FAILURE VALUE returns here

	// This writes out the comma-separated file that is
	// loadable in pretty much any spreadsheet
	fprintf(ofptr, "Hr,Min,Deg,Name,Type,Range,Desc,Max,Min,Period\n");
	while(rPtr != NULL)
	{
		fprintf(ofptr, "%hd,%hd,%hd,\"%s\",\"%s\",\"%s\",\"%s\",%f,%f,%f\n", 
			rPtr->pVSR->ra_hours,
			rPtr->pVSR->ra_minutes,
			rPtr->pVSR->dec_degrees,
			rPtr->pVSR->name,
			rPtr->pVSR->type,
			rPtr->pVSR->range,
			rPtr->pVSR->desc,
			rPtr->pVSR->max_mag,
			rPtr->pVSR->min_mag,
			rPtr->pVSR->period_days);

		i++;
		rPtr = rPtr->pNext;
	}

	fclose(ofptr);

	return i;
}

//--== saveFile() ==-----------------------------------------------------------
//
// Save current search results as a text file with records in original format
//
int saveFile(const char* fname)
{
	struct VarStarResult* rPtr = topResPtr;
	int i = 0;

	if(!is_loaded)
		return 0;

	// ".txt" appended to passed file name if necessary
	strcpy(rptname, fname);
	if(strstr(fname, ".txt") == NULL);
		strcat(rptname, ".txt");

	ofptr = fopen(rptname, "w");

	if(ofptr == NULL)
		return -1;	// FAILURE VALUE returns here

	// Write out raw records to output file
	while(rPtr != NULL)
	{
		fprintf(ofptr, "%s", rPtr->pVSR->record);
		i++;
		rPtr = rPtr->pNext;
	}

	fclose(ofptr);

	return i;
}

//--== isLineValidRecord() ==--------------------------------------------------
//
// Test buffer for valid record format
//
int isLineValidRecord(const char* testStr)
{
	//  4 bytes: R.A. as hhmm
	//  1 byte:  north or south as + or -
	//  2 bytes: dec. in whole degrees
	//  1 byte:  space or alpha character
	//  2 bytes: spaces
	//  1 byte:  printable non-space
	// 11 bytes: unchecked
	//  1 byte:  printable non-space

	int i;

	for(i = 0; i <= 3; i++)
		if(!isdigit(testStr[i]))
			return 0;

	if(!(testStr[4] == '+' || testStr[4] == '-'))
		return 0;

	for(i = 5; i <= 6; i++)
		if(!isdigit(testStr[i]))
			return 0;

	if((testStr[7] != ' ') 
		&& (!(testStr[7] >= 'A' && testStr[7] <= 'Z')))
			return 0;

	for(i = 8; i <= 9; i++)
		if(testStr[i] != ' ')
			return 0;
	
	if(!isgraph(testStr[10]) || !isgraph(testStr[22]))
		return 0;

	return 1;
}

//--== setHours() ==-----------------------------------------------------------
// Parse chars into integer list node member
void setHours(struct VarStarRecord* vPtr)
{
	vPtr->ra_hours = 0;
	vPtr->ra_hours +=
		((short int)(vPtr->record[0]) - 48) * 10;
	vPtr->ra_hours +=
		((short int)(vPtr->record[1]) - 48);
}

//--== setMinutes() ==---------------------------------------------------------
// Parse chars into integer list node memeber
void setMinutes(struct VarStarRecord* vPtr)
{
	vPtr->ra_minutes = 0;

	vPtr->ra_minutes +=
		((short int)(vPtr->record[2]) - 48) * 10;
	vPtr->ra_minutes +=
		((short int)(vPtr->record[3]) - 48);
}

//--== setDegrees() ==---------------------------------------------------------
// Parse chars into integer list node member
void setDegrees(struct VarStarRecord* vPtr)
{
	vPtr->dec_degrees = 0;

	vPtr->dec_degrees +=
		((short int)(vPtr->record[5]) - 48) * 10;
	vPtr->dec_degrees +=
		((short int)(vPtr->record[6]) - 48);
	if(vPtr->record[4] == '-')
		vPtr->dec_degrees = -(vPtr->dec_degrees);
}

//--== setName() ==------------------------------------------------------------
// Copy name into char* list node member - right aligned & space padded
void setName(struct VarStarRecord* vPtr)
{
	int i, j = 0;

	// determines number of spaces to pad from the left
	for(i = 20; i >= 10; i--)
	{
		if(vPtr->record[i] == ' ')
			j++;
		else
			break;
	}

	vPtr->name[11] = '\0';

	// fill in the name from the right
	for(i = 10 - j; i >= 0; i--)
		vPtr->name[i + j] = vPtr->record[i + 10];

	// fill in the spaces from the right
	for(i = 0; i < j; i++)
		vPtr->name[i] = ' ';
}

//--== setType() ==------------------------------------------------------------
// Copy name into char* list node member
void setType(struct VarStarRecord* vPtr)
{
	int i;

	for(i = 0; i < 8; i++)
		vPtr->type[i] = vPtr->record[i + 22];

	vPtr->type[8] = '\0';
}

//--== setRange() ==-----------------------------------------------------------
// Copy name into char* list node member
void setRange(struct VarStarRecord* vPtr)
{
	int i;

	for(i = 0; i < 14; i++)
		vPtr->range[i] = vPtr->record[i + 32];

	vPtr->range[14] = '\0';
}

//--== setMaxMag() ==----------------------------------------------------------
// Parse chars into floating point list node member
void setMaxMag(struct VarStarRecord* vPtr)
{
	vPtr->max_mag = 0;

	if(isdigit(vPtr->record[33]))
		vPtr->max_mag +=
		((float)(vPtr->record[33]) - 48) * 10;
	if(isdigit(vPtr->record[34]))
		vPtr->max_mag +=
		((float)(vPtr->record[34]) - 48);
	if(isdigit(vPtr->record[36]))
		vPtr->max_mag +=
		((float)(vPtr->record[36]) - 48) * 0.1f;
	if(isdigit(vPtr->record[37]))
		vPtr->max_mag +=
		((float)(vPtr->record[37]) - 48) * 0.01f;
}

//--== setMinMag() ==----------------------------------------------------------
// Parse chars into floating point list node member
void setMinMag(struct VarStarRecord* vPtr)
{
	vPtr->min_mag = 0;

	if(isdigit(vPtr->record[40]))
		vPtr->min_mag +=
		((float)(vPtr->record[40]) - 48) * 10;
	if(isdigit(vPtr->record[41]))
		vPtr->min_mag +=
		((float)(vPtr->record[41]) - 48);
	if(isdigit(vPtr->record[43]))
		vPtr->min_mag +=
		((float)(vPtr->record[43]) - 48) * 0.1f;
	if(isdigit(vPtr->record[44]))
		vPtr->min_mag +=
		((float)(vPtr->record[44]) - 48) * 0.01f;
}

//--== setDays() ==------------------------------------------------------------
// Parse chars into floating point list node member
void setDays(struct VarStarRecord* vPtr)
{
	vPtr->period_days = 0;

	if(isdigit(vPtr->record[46]))
		vPtr->period_days +=
		((double)(vPtr->record[46]) - 48) * 1000;
	if(isdigit(vPtr->record[47]))
		vPtr->period_days +=
		((double)(vPtr->record[47]) - 48) * 100;
	if(isdigit(vPtr->record[48]))
		vPtr->period_days +=
		((double)(vPtr->record[48]) - 48) * 10;
	if(isdigit(vPtr->record[49]))
		vPtr->period_days +=
		((double)(vPtr->record[49]) - 48);
	if(vPtr->record[50] == 'K')
		vPtr->period_days *= 1000;
	if(isdigit(vPtr->record[51]))
		vPtr->period_days +=
		((double)(vPtr->record[51]) - 48) * 0.1;
	if(isdigit(vPtr->record[52]))
		vPtr->period_days +=
		((double)(vPtr->record[52]) - 48) * 0.01;
	if(isdigit(vPtr->record[53]))
		vPtr->period_days +=
		((double)(vPtr->record[53]) - 48) * 0.001;
	if(isdigit(vPtr->record[54]))
		vPtr->period_days +=
		((double)(vPtr->record[54]) - 48) * 0.0001;
}

//--== setDesc() ==------------------------------------------------------------
// Copy name into char* list node member
void setDesc(struct VarStarRecord* vPtr)
{
	int i;

	for(i = 0; i < 20; i++)
		vPtr->desc[i] = vPtr->record[i + 55];

	vPtr->desc[20] = '\0';
}

//--== clearSearch() ==--------------------------------------------------------
//
// Destroy search result list, if any,
// and create a result list including all records
//
void clearSearch()
{
	// if file loaded, there will be at least one
	// so create first results node and create a
	// record pointer to walk through the main list
	struct VarStarRecord* recPtr;
	int i = 0;
	recPtr = topPtr;
	topResPtr = malloc(sizeof(struct VarStarResult));
	topResPtr->pPrev = NULL;
	resPtr = topResPtr;

	if(!is_loaded)
		return;

	if(is_searched)
	{
		delResults();
		is_searched = FALSE;
		is_specific = FALSE;
	}

   	// assign result node values until reaching
   	// the last main list node
	while(recPtr->pNext != NULL)
	{
		resPtr->recnum = ++i;
		resPtr->pVSR   = recPtr;
		resPtr->pNext  = malloc(sizeof(struct VarStarResult));
		resPtr->pNext->pPrev = resPtr;
		resPtr         = resPtr->pNext;

		recPtr = recPtr->pNext;
	}

	// assign values to the final result node
	resPtr->recnum = ++i;
	resPtr->pVSR   = recPtr;
	resPtr->pNext  = NULL;

	// reset search values to beginning of results
	lastResPtr = resPtr;
	resPtr = topResPtr;
	begin  = TRUE;
	end    = FALSE;
	is_searched = TRUE;
	is_specific = FALSE;
}

//--== searchRecords() ==------------------------------------------------------
//
// Calls specific or non-specific search & sets search state vars
// When a record meets search criteria, a results node is created
//
int searchRecords(int specific, int magnitude, int period)
{
	// most of the time, there will be at least one
	// result so create first results node and create a
	// record pointer to walk through the main list
	struct VarStarRecord* recPtr = topPtr;
	int i = 0;

	if(!is_loaded)
		return -1;

	if(is_searched)
	{
		delResults();
		is_searched = FALSE;
		is_specific = FALSE;
	}

   	// assign result node values until reaching
   	// the last main list node
	while(recPtr != NULL)
	{
		// call specific or non-specific search
		if( specific == 1
			? isRecordInSearch(recPtr, magnitude, period)
			: isRecordInSearchUncertain(recPtr) )
		{
			if(i == 0)
			{
				topResPtr = malloc(sizeof(struct VarStarResult));
				resPtr = topResPtr;
				resPtr->pPrev = NULL;
			}
			else
			{
				resPtr->pNext = malloc(sizeof(struct VarStarResult));
				resPtr->pNext->pPrev = resPtr;
				resPtr = resPtr->pNext;
			}

			resPtr->recnum = ++i;
			resPtr->pVSR   = recPtr;
		}

		recPtr = recPtr->pNext;
	}

	// if any results, assign values to final result

	if(i > 0)
	{
		// assign values to the final result node
		resPtr->pNext  = NULL;

		// reset search values to beginning of results
		lastResPtr = resPtr;
		resPtr     = topResPtr;
		begin  = TRUE;
		end    = FALSE;
		is_searched = TRUE;
		if(specific) 
			is_specific = TRUE; 
		else
			is_specific = FALSE;
		
		return i;
	}
	else // no results, so clean up
	{
		//delete resPtr;

		resPtr     = NULL;
		topResPtr  = NULL;
		lastResPtr = NULL;
		begin  = TRUE;
		end    = TRUE;
		is_searched = FALSE;
		is_specific = FALSE;
		
		return 0;
	}
}

//--== isRecordInSearch() ==---------------------------------------------------
//
// Check if record meets "specific" search criteria
//
int isRecordInSearch(struct VarStarRecord* vPtr, int magnitude, int period)
{
	// R.A. hours will "bridge" 0 hours and look from east to west
	// based on how both minimun and maximum are set no matter which
	// is the greater numeric value
	//
	// Max degrees is already enforced as greater than min degrees
	// with pos values being north and neg values being south
	//
	// Magnitude values NOT zero and...
	// Min magnitude >= value, so "no brighter than" (mag search tricky!)
	// Max magnitude <= value, so "no dimmer than"   (mag search tricky!)
	//
	// Period values NOT zero and...
	// Low and high limits on period in days
		if( (minHr <= maxHr
					? (vPtr->ra_hours >= minHr && vPtr->ra_hours <= maxHr)
					: (vPtr->ra_hours >= minHr || vPtr->ra_hours <= maxHr))
				&& vPtr->dec_degrees >= minDeg && vPtr->dec_degrees <= maxDeg
				&& ( magnitude
					? (	    vPtr->max_mag <= maxMag && vPtr->max_mag != 0
						&& (vPtr->min_mag >= minMag || vPtr->min_mag == 0))
					: TRUE )
				&& ( period
					? (	   vPtr->period_days >= minDays && vPtr->period_days <= maxDays
						&& vPtr->period_days != 0 )
					: TRUE ) )

			return TRUE;
		else
			return FALSE;
}

//--== isRecordInSearchUncertain() ==------------------------------------------
//
// Check if record meets "sky area only" search criteria
//
int isRecordInSearchUncertain(struct VarStarRecord* vPtr)
{
	// R.A. hours will "bridge" 0 hours and look from east to west
	// based on how both minimun and maximum are set no matter which
	// is the greater numeric value
	//
	// Max degrees is already enforced as greater than min degrees
	// with pos values being north and neg values being south
	//
	// NO magnitude or period information for records in this search
	// being mutually exclusive to the "specific" search within the
	// same sky area
	if( (minHr <= maxHr
			? (vPtr->ra_hours >= minHr  && vPtr->ra_hours <= maxHr)
			: (vPtr->ra_hours >= minHr  || vPtr->ra_hours <= maxHr))
		&& vPtr->dec_degrees >= minDeg  && vPtr->dec_degrees <= maxDeg
		&& vPtr->min_mag == 0
		&& vPtr->max_mag == 0
		&& vPtr->period_days == 0)

		return 1;
	else
		return 0;
}


//--== getHour() ==------------------------------------------------------------
// Return R.A. hour of current record or 0 if no results list
short int getHour()
{
	if(resPtr != NULL)
		return resPtr->pVSR->ra_hours;
	else
		return 0;
}

//--== getMin() ==-------------------------------------------------------------
// Return R.A. minute of current record or 0 if no results list
short int getMin ()
{
	if(resPtr != NULL)
		return resPtr->pVSR->ra_minutes;
	else
		return 0;
}

//--== getDeg() ==-------------------------------------------------------------
// Return declination degree of current record or 0 if no results list
short int getDeg ()
{
	if(resPtr != NULL)
		return resPtr->pVSR->dec_degrees;
	else
		return 0;
}

//--== getName() ==------------------------------------------------------------
// Return name of current record or empty string if no results list
const char* getName()
{
	if(resPtr != NULL)
		return resPtr->pVSR->name;
	else
		return "";
}

//--== getType() ==------------------------------------------------------------
// Return type of current record or empty string if no results list
const char* getType()
{
	if(resPtr != NULL)
		return resPtr->pVSR->type;
	else
		return "";
}

//--== getRange() ==-----------------------------------------------------------
// Return range string of current record or empty string if no results list
const char* getRange()
{
	if(resPtr != NULL)
		return resPtr->pVSR->range;
	else
		return "";
}

//--== getDays() ==------------------------------------------------------------
// Return period in days of current record or 0 if no results list
void getDays(char* per)
{
	if(resPtr != NULL && resPtr->pVSR->period_days != 0)
		sprintf(per, "%12.4lf", resPtr->pVSR->period_days);
	else
		strcpy(per, "");
}

//--== getDesc() ==------------------------------------------------------------
// Return description of current record or empty string if no results list
const char* getDesc()
{
	if(resPtr != NULL)
		return resPtr->pVSR->desc;
	else
		return "";
}

//--== getRange() ==-----------------------------------------------------------
// Return raw record of current record or empty string if no results list
const char* getRecord()
{
	if(resPtr != NULL)
		return resPtr->pVSR->record;
	else
		return "";
}

//--== moveFirst() ==----------------------------------------------------------
// Move results pointer to the top of the results list
void moveFirst()
{
	if(!is_searched)
		return;
	resPtr = topResPtr;
	begin = TRUE;
	end = FALSE;
}

//--== movePrev() ==-----------------------------------------------------------
// Move results pointer to the previous node unless at the top
void movePrev()
{
	if(!is_searched)
		return;
	if(resPtr->pPrev == NULL)
		begin = TRUE;
	else 
		begin = FALSE;
	if(resPtr->pPrev != NULL)
		resPtr = resPtr->pPrev;

	end = FALSE;
}

//--== moveNext() ==-----------------------------------------------------------
// Move results pointer to the next node unless at the end
void moveNext()
{
	if(!is_searched)
		return;
	if(resPtr->pNext == NULL)
		end = TRUE;
	else
		end = FALSE;
	if(resPtr->pNext != NULL)
		resPtr = resPtr->pNext;

	begin = FALSE;
}

//--== moveLast() ==-----------------------------------------------------------
// Move results pointer to the end of the results list
void moveLast()
{
	if(!is_searched)
		return;
	resPtr = lastResPtr;
	begin = FALSE;
	end = TRUE;

}

//--== bof() ==----------------------------------------------------------------
// Am I at the top of the results list?
int bof()
{
	return begin;
}

//--== eof() ==----------------------------------------------------------------
// Am I at the end of the results list?
int eof()
{
	return end;
}

//--== isLoaded() ==-----------------------------------------------------------
// What is the loaded state?
int isLoaded()
{
	return is_loaded;
}

//--== isSearched() ==---------------------------------------------------------
// What is the searched state?
int isSearched()
{
	return is_searched;
}

//--== delResults() ==---------------------------------------------------------
//
// Destroy search result list if it exists
//
void delResults()
{
	struct VarStarResult* delResPtr = topResPtr;
	struct VarStarResult* recResPtr = topResPtr->pNext;

	// if top of list NULL, loop does not execute
	while(delResPtr != NULL)
	{
		free((void*)delResPtr);

		// if only one result node, delete it,
		// NULL necessary pointers, quit
		if(recResPtr == NULL) 
		{
			delResPtr = NULL;

			topResPtr  = NULL;
			resPtr     = NULL;
			lastResPtr = NULL;

			break;
		}

		// if yet another node, move pointers
		if(recResPtr->pNext != NULL)
		{
			delResPtr = recResPtr;
			recResPtr = recResPtr->pNext;
		}
		else
		{
			free((void*)recResPtr);
			delResPtr = NULL;
			recResPtr = NULL;

			topResPtr  = NULL;
			resPtr     = NULL;
			lastResPtr = NULL;
		}
	}
}

//--== delRecords() ==---------------------------------------------------------
//
// Destroy main records list if it exists
//
void delRecords()
{
	struct VarStarRecord* delPtr = topPtr;
	struct VarStarRecord* recPtr = topPtr->pNext;

	// if top of list NULL, loop does not execute
	while(delPtr != NULL)
	{
		free((void*)delPtr);

		// if only one record node, delete it,
		// NULL necessary pointers, quit
		if(recPtr == NULL) 
		{
			delPtr = NULL;

			topPtr  = NULL;
			lastPtr = NULL;

			break;
		}

		// if yet another node, move pointers
		if(recPtr->pNext != NULL)
		{
			delPtr = recPtr;
			recPtr = recPtr->pNext;
		}
		else
		{
			free((void*)recPtr);
			delPtr = NULL;
			recPtr = NULL;

			topPtr  = NULL;
			lastPtr = NULL;
		}
	}
}

// May use these funcs rather than #include <ctype.h>
/*
int isdit(char chr)
{
	return (int)(chr >= '0' && chr <= '9');
}

int isgph(char chr)
{
	return (int)(chr >= '!' && chr <= '~');
}
*/

